//#=====================================================================================
//#
//#       Filename:  main.cpp
//#
//#    Description:  Balance~ PD/Max external main code.
//#
//#        Version:  1.0
//#        Created:  01/09/03
//#       Revision:  none
//#
//#         Author:  Frank Barknecht  (fbar)
//#          Email:  fbar@footils.org
//#      Copyright:  Frank Barknecht , 2003
//#
//#        This program is free software; you can redistribute it and/or modify
//#    it under the terms of the GNU General Public License as published by
//#    the Free Software Foundation; either version 2 of the License, or
//#    (at your option) any later version.
//#
//#    This program is distributed in the hope that it will be useful,
//#    but WITHOUT ANY WARRANTY; without even the implied warranty of
//#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//#    GNU General Public License for more details.
//#
//#    You should have received a copy of the GNU General Public License
//#    along with this program; if not, write to the Free Software
//#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//#
//#
//#=====================================================================================



// include flext header
#include <flext.h>
#include <flsndobj.h>
#include <SndObj/AudioDefs.h>
// check for appropriate flext version
#if !defined(FLEXT_VERSION) || (FLEXT_VERSION < 401)
#error You need at least flext version 0.4.1
#endif


class balance:
	public flext_sndobj
{
	FLEXT_HEADER(balance,flext_sndobj)
 
public:
	// constructor 
	balance(int argc, t_atom *argv);
	// destroy
	// ~balance();
	virtual bool NewObjs();
	virtual void FreeObjs();
	virtual void ProcessObjs();
	virtual void m_help();
private:
	Balance *bal;
	float fr;
	
};

// instantiate the class 
FLEXT_NEW_DSP_V("balance~",balance)


balance::balance(int argc, t_atom *argv)
	:  // initialize arguments
	fr(10.f), bal(NULL)
	
{ 
	// define inlets
	AddInSignal(2);
	AddOutSignal();
} 
bool balance::NewObjs()
{
	// set up objects
	bal = new Balance( &InObj(0), &InObj(1), fr, Blocksize(), Samplerate() );
	return true;
}

// destroy the SndObjs
void balance::FreeObjs()
{
	if (bal)	delete bal;
}

// this is called on every DSP block
void balance::ProcessObjs()
{
	bal->DoProcess();
	// output
	*bal >> OutObj(0);
}


void balance::m_help()
{
	post("");
	post("_ _____balance~ help___ _");
	post("_ _____balances the amplitude of left signal with that of right signal");
	post("This object balances two signals: it measures the rms power of");
	post("the second input signal (the control signal), by a combination of");
	post("rectification and low-pass filtering, and uses that information");
	post("to control the gain of the first signal.  This signal is then fed");
	post("to the output of the object. The other parameter used by the");
	post("object is the frequency of the internal low-pass filter.");
}
	
